@@ -19,7 +19,7 @@
  */
 
 
-/* $Id: mhash.c,v 1.9 2000/10/20 14:32:22 nmav Exp $ */
+/* $Id: mhash.c,v 1.13 2000/12/15 12:42:33 nmav Exp $ */
 
 #include <stdlib.h>
 
@@ -73,8 +73,7 @@ static mhash_hash_entry algorithms[] = {
 #define MHASH_ALG_LOOP(a) \
 	MHASH_LOOP( if(p->id == type) { a; break; } )
 
-WIN32DLL_DEFINE
-size_t mhash_count(void)
+WIN32DLL_DEFINE size_t mhash_count(void)
 {
 	size_t count = 0;
 
@@ -83,8 +82,7 @@ size_t mhash_count(void)
 	return count;
 }
 
-WIN32DLL_DEFINE
-size_t mhash_get_block_size(hashid type)
+WIN32DLL_DEFINE size_t mhash_get_block_size(hashid type)
 {
 	size_t ret = 0;
 
@@ -94,9 +92,10 @@ size_t mhash_get_block_size(hashid type)
 
 #ifdef WIN32
 /* function created in order for mhash to compile under WIN32 */
-static char* mystrdup(char* str) {
-char* ret;
-	ret = malloc(strlen(str)+1);
+char *mystrdup(char *str)
+{
+	char *ret;
+	ret = malloc(strlen(str) + 1);
 	strcpy(ret, str);
 
 	return ret;
@@ -104,21 +103,37 @@ char* ret;
 }
 #endif
 
-WIN32DLL_DEFINE
-char *mhash_get_hash_name(hashid type)
+WIN32DLL_DEFINE hashid mhash_get_mhash_algo( MHASH tmp) {
+	return tmp->algorithm_given;
+}
+
+WIN32DLL_DEFINE char *mhash_get_hash_name(hashid type)
 {
 	char *ret = NULL;
 
 	/* avoid prefix */
-#ifdef WIN32
 	MHASH_ALG_LOOP(ret = mystrdup(p->name + sizeof("MHASH_") - 1));
-#else
-	MHASH_ALG_LOOP(ret = strdup(p->name + sizeof("MHASH_") - 1));
-#endif
 	return ret;
 }
 
+MHASH mhash_cp(MHASH from) {
+MHASH ret;
 
+	ret = malloc(sizeof(MHASH_INSTANCE));
+	memcpy(ret, from, sizeof(MHASH_INSTANCE));
+	
+	/* copy the internal state also */
+	ret->state=malloc(ret->state_size);
+	memcpy( ret->state, from->state, ret->state_size);
+	
+	/* copy the key in case of hmac*/
+	if (ret->hmac_key_size!=0) {
+		ret->hmac_key=malloc(ret->hmac_key_size);
+		memcpy(ret->hmac_key, from->hmac_key, ret->hmac_key_size);
+	}
+	return ret;
+
+}
 MHASH mhash_init_int(const hashid type)
 {
 	MHASH ret;
@@ -128,57 +143,69 @@ MHASH mhash_init_int(const hashid type)
 	ret->algorithm_given = type;
 	ret->hmac_key = NULL;
 	ret->state = NULL;
-
+	ret->hmac_key_size = 0;
+	
 	switch (type) {
 	case MHASH_CRC32:
 	case MHASH_CRC32B:
-		ret->state = malloc(sizeof(word32));
+		ret->state_size = sizeof(word32);
+		ret->state = malloc(ret->state_size);
 		clear_crc32((void *) ret->state);
 		break;
 	case MHASH_MD5:
-		ret->state = malloc(sizeof(MD5_CTX));
+		ret->state_size = sizeof(MD5_CTX);
+		ret->state = malloc(ret->state_size);
 		MD5Init((void *) ret->state);
 		break;
 	case MHASH_SHA1:
-		ret->state = malloc(sizeof(SHA_CTX));
+		ret->state_size = sizeof(SHA_CTX);
+		ret->state = malloc(ret->state_size);
 		sha_init((void *) ret->state);
 		break;
 	case MHASH_HAVAL256:
-		ret->state = malloc(sizeof(havalContext));
-		havalInit( (void *) ret->state, 3, 256);
+		ret->state_size = sizeof(havalContext);
+		ret->state = malloc(ret->state_size);
+		havalInit((void *) ret->state, 3, 256);
 		break;
 	case MHASH_HAVAL224:
-		ret->state = malloc(sizeof(havalContext));
-		havalInit( (void *) ret->state, 3, 224);
+		ret->state_size = sizeof(havalContext);
+		ret->state = malloc(ret->state_size);
+		havalInit((void *) ret->state, 3, 224);
 		break;
 	case MHASH_HAVAL192:
-		ret->state = malloc(sizeof(havalContext));
-		havalInit( (void *) ret->state, 3, 192);
+		ret->state_size = sizeof(havalContext);
+		ret->state = malloc(ret->state_size);
+		havalInit((void *) ret->state, 3, 192);
 		break;
 	case MHASH_HAVAL160:
-		ret->state = malloc(sizeof(havalContext));
-		havalInit( (void *) ret->state, 3, 160);
+		ret->state_size = sizeof(havalContext);
+		ret->state = malloc(ret->state_size);
+		havalInit((void *) ret->state, 3, 160);
 		break;
 	case MHASH_HAVAL128:
-		ret->state = malloc(sizeof(havalContext));
-		havalInit( (void *) ret->state, 3, 128);
+		ret->state_size = sizeof(havalContext);
+		ret->state = malloc(ret->state_size);
+		havalInit((void *) ret->state, 3, 128);
 		break;
 	case MHASH_RIPEMD160:
-		ret->state = malloc(sizeof(RIPEMD_CTX));
+		ret->state_size = sizeof(RIPEMD_CTX);
+		ret->state = malloc(ret->state_size);
 		ripemd_init((void *) ret->state);
 		break;
 	case MHASH_TIGER:
-		ret->state = malloc(3 * sizeof(word64));
+		ret->state_size = 3 * sizeof(word64);
+		ret->state = malloc(ret->state_size);
 		break;
 	case MHASH_GOST:
-		ret->state = malloc(sizeof(GostHashCtx));
+		ret->state_size = sizeof(GostHashCtx);
+		ret->state = malloc(ret->state_size);
 		gosthash_reset((void *) ret->state);
 		break;
 	default:
 		ret = MHASH_FAILED;
 		break;
 	}
-	
+
 	return ret;
 }
 
@@ -244,7 +271,7 @@ int mhash(MHASH thread, const void *plaintext, size_t size)
 }
 
 WIN32DLL_DEFINE
-void *mhash_end(MHASH thread)
+    void *mhash_end_m(MHASH thread, void *(*hash_malloc) (size_t))
 {
 	void *digest;
 	void *rtmp = NULL;
@@ -252,17 +279,19 @@ void *mhash_end(MHASH thread)
 	switch (thread->algorithm_given) {
 	case MHASH_CRC32:
 	case MHASH_CRC32B:
-		rtmp = get_crc32((void *) thread->state);
+		rtmp = hash_malloc(sizeof(word32));
+		get_crc32(rtmp, (void *) thread->state);
 		break;
 	case MHASH_MD5:
 		digest =
-		    malloc(mhash_get_block_size(thread->algorithm_given));
+		    hash_malloc(mhash_get_block_size
+				(thread->algorithm_given));
 		MD5Final(digest, (void *) thread->state);
-		rtmp=digest;
+		rtmp = digest;
 		break;
 	case MHASH_SHA1:
 		sha_final((void *) thread->state);
-		digest = malloc(SHA_DIGESTSIZE);
+		digest = hash_malloc(SHA_DIGESTSIZE);
 		sha_digest((void *) thread->state, digest);
 		rtmp = digest;
 		break;
@@ -272,24 +301,25 @@ void *mhash_end(MHASH thread)
 	case MHASH_HAVAL160:
 	case MHASH_HAVAL128:
 		digest =
-		    malloc(mhash_get_block_size(thread->algorithm_given));
+		    hash_malloc(mhash_get_block_size
+				(thread->algorithm_given));
 		havalFinal((void *) thread->state, digest);
 		rtmp = digest;
 		break;
 	case MHASH_RIPEMD160:
 		ripemd_final((void *) thread->state);
-		digest = malloc(RIPEMD_DIGESTSIZE);
+		digest = hash_malloc(RIPEMD_DIGESTSIZE);
 		ripemd_digest((void *) thread->state, digest);
 		rtmp = digest;
 		break;
 	case MHASH_TIGER:
-		digest = malloc(192 >> 3);
+		digest = hash_malloc(192 >> 3);
 		memcpy(digest, (void *) thread->state, 192 >> 3);
 		mhash_32bit_conversion(digest, 192 >> 5);
 		rtmp = digest;
 		break;
 	case MHASH_GOST:
-		digest = malloc(32);
+		digest = hash_malloc(32);
 		gosthash_final((void *) thread->state, digest);
 		rtmp = digest;
 		break;
@@ -303,22 +333,16 @@ void *mhash_end(MHASH thread)
 	return rtmp;
 }
 
-WIN32DLL_DEFINE
-MHASH mhash_init(const hashid type)
+WIN32DLL_DEFINE void *mhash_end(MHASH thread)
+{
+	return mhash_end_m(thread, malloc);
+}
+
+
+WIN32DLL_DEFINE MHASH mhash_init(const hashid type)
 {
-	static int gost_init, crc32b_init;
 	MHASH ret = MHASH_FAILED;
 
-	if (type == MHASH_GOST && !gost_init) {
-		gosthash_init();
-		gost_init++;
-	}
-
-	if (type == MHASH_CRC32B && !crc32b_init) {
-		crc32b_init++;
-		crc32bgen();
-	}
-
 	ret = mhash_init_int(type);
 
 	return ret;
@@ -326,8 +350,7 @@ MHASH mhash_init(const hashid type)
 
 /* HMAC functions */
 
-WIN32DLL_DEFINE
-size_t mhash_get_hash_pblock(hashid type)
+WIN32DLL_DEFINE size_t mhash_get_hash_pblock(hashid type)
 {
 	size_t ret = 0;
 
@@ -335,8 +358,9 @@ size_t mhash_get_hash_pblock(hashid type)
 	return ret;
 }
 
+
 WIN32DLL_DEFINE
-void *mhash_hmac_end(MHASH thread)
+    void *mhash_hmac_end_m(MHASH thread, void *(*hash_malloc) (size_t))
 {
 	void *digest;
 	unsigned char *opad;
@@ -359,16 +383,19 @@ void *mhash_hmac_end(MHASH thread)
 	switch (thread->algorithm_given) {
 	case MHASH_CRC32:
 	case MHASH_CRC32B:
-		return_val = get_crc32((void *) thread->state);
+		digest = hash_malloc(sizeof(word32));
+		get_crc32(digest, (void *) thread->state);
+		return_val = digest;
 		break;
 	case MHASH_MD5:
 		digest =
-		    malloc(mhash_get_block_size(thread->algorithm_given));
+		    hash_malloc(mhash_get_block_size
+				(thread->algorithm_given));
 		MD5Final(digest, (void *) thread->state);
 		return_val = digest;
 		break;
 	case MHASH_SHA1:
-		digest = malloc(SHA_DIGESTSIZE);
+		digest = hash_malloc(SHA_DIGESTSIZE);
 		sha_final((void *) thread->state);
 		sha_digest((void *) thread->state, digest);
 		return_val = digest;
@@ -379,23 +406,24 @@ void *mhash_hmac_end(MHASH thread)
 	case MHASH_HAVAL160:
 	case MHASH_HAVAL128:
 		digest =
-		    malloc(mhash_get_block_size(thread->algorithm_given));
+		    hash_malloc(mhash_get_block_size
+				(thread->algorithm_given));
 		havalFinal((void *) thread->state, digest);
 		return_val = digest;
 		break;
 	case MHASH_RIPEMD160:
-		digest = malloc(RIPEMD_DIGESTSIZE);
+		digest = hash_malloc(RIPEMD_DIGESTSIZE);
 		ripemd_final((void *) thread->state);
 		ripemd_digest((void *) thread->state, digest);
 		return_val = digest;
 		break;
 	case MHASH_TIGER:
-		digest = malloc(192 >> 3);
+		digest = hash_malloc(192 >> 3);
 		memcpy(digest, (void *) thread->state, 192 >> 3);
 		return_val = digest;
 		break;
 	case MHASH_GOST:
-		digest = malloc(32);
+		digest = hash_malloc(32);
 		gosthash_final((void *) thread->state, digest);
 		return_val = digest;
 		break;
@@ -407,20 +435,26 @@ void *mhash_hmac_end(MHASH thread)
 	if (NULL != return_val) {
 		free(return_val);
 	}
-	
+
 	free(thread->state);
 	free(opad);
-	mhash_bzero( thread->hmac_key, thread->hmac_key_size);
+	mhash_bzero(thread->hmac_key, thread->hmac_key_size);
 	free(thread->hmac_key);
 	free(thread);
 
 	return mhash_end(tmptd);
 }
 
+WIN32DLL_DEFINE void *mhash_hmac_end(MHASH thread)
+{
+	return mhash_hmac_end_m(thread, malloc);
+}
+
+
 WIN32DLL_DEFINE
-MHASH mhash_hmac_init(const hashid type, void *key, int keysize, int block)
+    MHASH mhash_hmac_init(const hashid type, void *key, int keysize,
+			  int block)
 {
-	static int gost_init, crc32b_init;
 	MHASH ret = MHASH_FAILED;
 	MHASH tmptd;
 	unsigned char *tmp;
@@ -431,57 +465,45 @@ MHASH mhash_hmac_init(const hashid type, void *key, int keysize, int block)
 		block = 64;	/* the default for ripemd,md5,sha-1 */
 	}
 
-	if (type == MHASH_GOST && !gost_init) {
-		gost_init++;
-		gosthash_init();
-	}
+	ret = mhash_init_int(type);
 
-	if (type == MHASH_CRC32B && !crc32b_init) {
-		crc32b_init++;
-		crc32bgen();
-	}
+	if (ret != MHASH_FAILED) {
+		/* Initial hmac calculations */
+		ret->hmac_block = block;
 
+		ipad = malloc(ret->hmac_block);
 
-		ret = mhash_init_int(type);
-
-		if (ret != MHASH_FAILED) {
-			/* Initial hmac calculations */
-			ret->hmac_block = block;
-
-			ipad = malloc(ret->hmac_block);
-
-			if (keysize > ret->hmac_block) {
-				tmptd = mhash_init(type);
-				mhash(tmptd, key, keysize);
-				ret->hmac_key_size =
-				    mhash_get_block_size(type);
-				ret->hmac_key = mhash_end(tmptd);
-			} else {
-				ret->hmac_key = calloc(1, ret->hmac_block);
-				memmove(ret->hmac_key, key, keysize);
-				ret->hmac_key_size = ret->hmac_block;
-			}
-
-			/* IPAD */
-
-			for (i = 0; i < ret->hmac_key_size; i++) {
-				ipad[i] = (0x36) ^ ret->hmac_key[i];
-			}
-			for (;i<ret->hmac_block;i++) {
-				ipad[i] = (0x36);
-			}
-
-			mhash(ret, ipad, ret->hmac_block);
-			
-			free(ipad);
+		if (keysize > ret->hmac_block) {
+			tmptd = mhash_init(type);
+			mhash(tmptd, key, keysize);
+			ret->hmac_key_size = mhash_get_block_size(type);
+			ret->hmac_key = mhash_end(tmptd);
+		} else {
+			ret->hmac_key = calloc(1, ret->hmac_block);
+			memmove(ret->hmac_key, key, keysize);
+			ret->hmac_key_size = ret->hmac_block;
 		}
 
+		/* IPAD */
+
+		for (i = 0; i < ret->hmac_key_size; i++) {
+			ipad[i] = (0x36) ^ ret->hmac_key[i];
+		}
+		for (; i < ret->hmac_block; i++) {
+			ipad[i] = (0x36);
+		}
+
+		mhash(ret, ipad, ret->hmac_block);
+
+		free(ipad);
+	}
+
 
 
 	return ret;
 }
 
-WIN32DLL_DEFINE
-void mhash_free(void * ptr) {
+WIN32DLL_DEFINE void mhash_free(void *ptr)
+{
 	free(ptr);
 }
